local cutscene = require(mod_name .. '.lualib.cutscene')
local locations = require(mod_name .. '.lualib.locations')
local colony_controller = require(mod_name .. '.lualib.colony_controller')
local eastern_cutscene_biters = require(mod_name .. '.lualib.npe.eastern_cutscene_biters')
local story = require("story_2")


local east_flank_worms = {}

 east_flank_worms.generate_cutscene = function()
  local story_nodes =
  {
    {
      name = "east_flank_worms-init",
      init = function()

        local east_worm_biters = eastern_cutscene_biters.get_east_flank_worm_group(7)
        east_worm_biters.set_command {
          type = defines.command.compound,
          structure_type = defines.compound_command.return_last,
          commands = {
            {
              type = defines.command.go_to_location,
              destination = locations.get_pos('cutscene-east-worms'),
              radius = 1,
              distraction = defines.distraction.none,
            },
            {
              type = defines.command.wander,
              radius = 1,
              distraction = defines.distraction.none,
            }
          }
        }

        local zoom = 0.5

        local scene =
        {
          type=defines.controllers.cutscene,
          waypoints=
          {
            {
              position = main_player().position,
              zoom = zoom,
              transition_time = 60 * 1.5,
              time_to_wait = 0,
              name = "pan_to_player"
            },
            {
              position = "target-4",
              transition_time = 60 * 1,
              time_to_wait = 60 * 1,
              zoom = 1,
              name = "copper_patch_view_one"
            },
            {
              position = "cutscene-east-worms",
              transition_time = 60 * 2,
              time_to_wait = 60 * 8,
              zoom = 1,
              name = "start_attack"
            },
            {
              position = main_player().position,
              transition_time = 60 * 1.5,
              time_to_wait = 0,
              zoom = zoom
            },
          },

          final_transition_time = 60 * 1.5
        }

        cutscene.play(scene)
      end,

      condition = function()
        return cutscene.get_last_waypoint_reached_name() == "copper_patch_view_one"
      end
    },
    {
      name = " east_flank_worms-start_attack",
      init = function()
        local group = eastern_cutscene_biters.get_east_flank_worm_group(7)
        if group.valid then
          for _, biter in pairs(group.members) do
            colony_controller.send_to_area(biter,locations.get_area('colony-east-mid'),1200)
          end
        end
      end,
      condition = function()
        local biters_in_target = locations.get_main_surface().find_entities_filtered({
          name = 'small-biter',
          area = locations.get_area('colony-east-mid')
        })
        return not cutscene.is_any_cutscene_playing() or #biters_in_target > 5 or
          cutscene.get_last_waypoint_reached_name() == "start_attack"
      end,
    },
    {
      name = "east_flank_worms-wait-a-second",
      condition = function()
        return story.check_seconds_passed('main_story',1)
      end,
      action = function()
        colony_controller.register('colony-east-mid',1,1,'enemy')
        local value = colony_controller.kill_all_units_in_colony('colony-east-mid')
        colony_controller.instant_spend_on_colony('colony-east-mid',value)
      end
    },
    {
      name = " east_flank_worms-final",
      condition = function()
        return not cutscene.is_any_cutscene_playing()
      end,
      action = function()
        colony_controller.register('colony-east-top',1,0,'enemy')
        colony_controller.register('colony-east-bottom',1,0,'enemy')
      end
    }
  }

  return story_nodes
end

return  east_flank_worms